@@ -243,7 +243,7 @@ int handle_status_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *para
 		print_single_value_int(stdout, params, "Total sessions", rep->total_sessions_closed, 1);
 		print_single_value_int(stdout, params, "Total authentication failures", rep->total_auth_failures, 1);
 		print_single_value_int(stdout, params, "IPs in ban list", rep->banned_ips, 1);
-		if (params->debug) {
+		if (params && params->debug) {
 			print_single_value_int(stdout, params, "Sec-mod client entries", rep->secmod_client_entries, 1);
 			print_single_value_int(stdout, params, "TLS DB entries", rep->stored_tls_sessions, 1);
 		}
@@ -1313,11 +1313,11 @@ int handle_events_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *para
 	uint32_t slength;
 	unsigned data_size;
 	uint8_t *data = NULL;
-	char *groupname;
 	char tmpbuf[MAX_TMPSTR_SIZE];
 	PROTOBUF_ALLOCATOR(pa, ctx);
 	struct termios tio_old, tio_new;
 	SIGHANDLER_T old_sighandler;
+	fd_set rfds;
 
 	init_reply(&raw);
 
@@ -1346,10 +1346,11 @@ int handle_events_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *para
 
 	/* start listening for updates */
 	while(1) {
-		fd_set rfds;
-
 		FD_ZERO(&rfds);
+#ifndef __clang_analyzer__
+		/* for some reason this confuses the clang static analyzer */
 		FD_SET(STDIN_FILENO, &rfds);
+#endif
 		FD_SET(ctx->fd, &rfds);
 
 		ret = select(MAX(STDIN_FILENO,ctx->fd)+1, &rfds, NULL, NULL, NULL);
@@ -1428,10 +1429,6 @@ int handle_events_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *para
 		if (HAVE_JSON(params)) {
 			common_info_cmd(rep2->user, stdout, params);
 		} else {
-			groupname = rep2->user->user[0]->groupname;
-			if (groupname == NULL || groupname[0] == 0)
-				groupname = NO_GROUP;
-
 			if (rep2->connected) {
 				printf("connected user '%s' (%u) from %s with IP %s\n",
 					rep2->user->user[0]->username,
