@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2016 Nikos Mavrogiannopoulos
+ * Copyright (C) 2013-2017 Nikos Mavrogiannopoulos
  * Copyright (C) 2014-2016 Red Hat
  *
  * This program is free software; you can redistribute it and/or modify
@@ -109,6 +109,23 @@ void sec_mod_add_score_to_ip(sec_mod_st *sec, client_entry_st *e, const char *ip
 	return;
 }
 
+static void update_auth_time_stats(sec_mod_st * sec, time_t secs)
+{
+	if (secs < 0)
+		return;
+
+	sec->total_authentications++;
+	if (sec->total_authentications == 0) { /* reset stats */
+		sec->avg_auth_time = 0;
+		sec->max_auth_time = 0;
+		return;
+	}
+
+	if (secs > sec->max_auth_time)
+		sec->max_auth_time = secs;
+	sec->avg_auth_time = (sec->avg_auth_time*(sec->total_authentications-1)+secs) / sec->total_authentications;
+}
+
 static
 int send_sec_auth_reply(int cfd, sec_mod_st * sec, client_entry_st * entry, AUTHREP r)
 {
@@ -125,6 +142,9 @@ int send_sec_auth_reply(int cfd, sec_mod_st * sec, client_entry_st * entry, AUTH
 			msg.msg = entry->msg_str;
 		}
 
+		/* calculate time in auth for this client */
+		update_auth_time_stats(sec, time(0) - entry->time);
+
 		msg.has_sid = 1;
 		msg.sid.data = entry->sid;
 		msg.sid.len = sizeof(entry->sid);
@@ -139,6 +159,8 @@ int send_sec_auth_reply(int cfd, sec_mod_st * sec, client_entry_st * entry, AUTH
 			       sec_auth_reply_msg__get_packed_size,
 			       (pack_func) sec_auth_reply_msg__pack);
 	} else {
+		sec->auth_failures++;
+
 		msg.reply = AUTH__REP__FAILED;
 
 		ret = send_msg(entry, cfd, CMD_SEC_AUTH_REPLY,
@@ -546,13 +568,8 @@ int handle_secm_session_close_cmd(sec_mod_st *sec, int fd, const SecmSessionClos
 	/* send reply */
 	rep.bytes_in = e->stats.bytes_in;
 	rep.bytes_out = e->stats.bytes_out;
-	rep.has_secmod_client_entries = 1;
 	rep.has_discon_reason = 1;
 	rep.discon_reason = e->discon_reason;
-	rep.secmod_client_entries = sec_mod_client_db_elems(sec);
-
-	rep.secmod_tlsdb_entries = sec->tls_db.entries;
-	rep.has_secmod_tlsdb_entries = 1;
 
 	ret = send_msg(e, fd, CMD_SECM_CLI_STATS, &rep,
 			(pack_size_func) cli_stats_msg__get_packed_size,
