@@ -33,4 +33,7 @@ struct tun_lease_st {
 	int fd;
 };
 
+ssize_t tun_write(int sockfd, const void *buf, size_t len);
+ssize_t tun_read(int sockfd, void *buf, size_t len);
+
 #endif
